<?php
// $Id: theme.inc,v 1.1.4.1 2010/12/25 22:42:21 pounard Exp $

/**
 * @file
 * XoXo theme and template preprocess functions.
 */

/**
 * Implementation of template_preprocess_TEMPLATE().
 */
function template_preprocess_xoxo_object(&$variables = array()) {
  $object = $variables['object'];

  // Check object is suitable for this theme function.
  if (!$object instanceof IRenderable) {
    $variables['object'] = '';
    $variables['classes'] = array();
    return;
  }

  $type = $object->getType();
  $factory = NULL;
  $identifier = $object->getIdentifier();
  $name = NULL;

  if ($object instanceof XoxoExportableObject) {
    try {
      $name = $object->getName();
      $factory = $object->getFactory();
    }
    // Silent error, object may not have been saved yet.
    catch (OoxException $e) { }
  }

  $suggestions = $classes = array();

  // All possible suggestions, this will allow great power to themer.
  if ($identifier) {
    $suggestions[] = 'xoxo__object__' . $type . '__' . $identifier;
    $classes[] = 'xoxo-object-' . $type . '-' . $identifier;
  }
  if ($name) {
    $suggestions[] = 'xoxo__object__' . $type . '__' . $name;
    $classes[] = 'xoxo-object-' . $type . '-' . $name;
  }
  $suggestions[] = 'xoxo__object__' . $type;
  $classes[] = 'xoxo-object-' . $type;

  // Add custom classes and theme suggestions for our template.
  $variables['attributes']['class'] = $classes;
  $variables['theme_hook_suggestions'] = $suggestions;

  // Eventually add contextual links if enabled. Any overriding custom module
  // can disable them if needed by unsetting the variable in their preprocess
  // implementation (by overriding this one).
  // Object may not have a factory, in some extreme cases.
  if ($factory) {
    $variables['#contextual_links']['xoxo_object'] = array($factory->getPath(), array($name ? $name : $identifier));
  }

  // Content.
  if (!empty($variables['content'])) {
    // This is the particular case where the module calling the theme function
    // already rendered the object, and passes it through the content argument.
    // In this case, we assume this is the already rendered object and skip the
    // render phase. Don't mess with this parameter unless you really now what
    // you are doing.
    // See the hook_theme() implementation for further details.
  }
  else if ($variables['preview']) {
    $variables['content'] = $object->renderPreview();
  }
  else {
    $variables['content'] = $object->render();
  }
}

/**
 * Implementation of template_process_TEMPLATE().
 */
function template_process_xoxo_object(&$variables = array()) {
  // FIXME: Weird bug here?
  $variables['attributes'] = is_array($variables['attributes']) ? drupal_attributes($variables['attributes']) : '';
}
